local pairs = pairs;
local type = type;
local tostring = tostring;
local tonumber = tonumber;
local strlen = strlen;
local strsub = strsub;
local strfind = strfind;
local floor = floor;


-- Sabc=S5|abcde
-- Sabc=N3|1.3
-- Sabc=0
-- Sabc=1
-- Sabc=T1234|...

-- Nabc=S1|x


local VUHDO_KEY_TO_ABBREV = {
	["useBackground"] = "*b",
	["color"] = "*c",
	["isStacks"] = "*d",
	["isIcon"] = "*e",
	["isColor"] = "*f",
	["bright"] = "*g",
	["others"] = "*h",
	["icon"] = "*i",
	["timer"] = "*j",
	["animate"] = "*k",
	["mine"] = "*m",
	["name"] = "*n",
	["useOpacity"] = "*o",
	["radio"] = "*r",
	["isManuallySet"] = "*s",
	["useText"] = "*t",
	["custom"] = "*u",
};


local VUHDO_ABBREV_TO_KEY = {
	["*b"] = "useBackground",
	["*c"] = "color",
	["*d"] = "isStacks",
	["*e"] = "isIcon",
	["*f"] = "isColor",
	["*g"] = "bright",
	["*h"] = "others",
	["*i"] = "icon",
	["*j"] = "timer",
	["*k"] = "animate",
	["*m"] = "mine",
	["*n"] = "name",
	["*o"] = "useOpacity",
	["*r"] = "radio",
	["*s"] = "isManuallySet",
	["*t"] = "useText",
	["*u"] = "custom",
};

--
function VUHDO_serializeTable(aTable)
	local tKey, tValue;
	local tString = "";

	for tKey, tValue in pairs(aTable) do
		if("number" == type(tKey)) then
			tString = tString .. "N" .. tKey .. "=";
		else
		  tKey = VUHDO_KEY_TO_ABBREV[tKey] or tKey;
			tString = tString .. "S" .. tKey .. "=";
		end

		if ("string" == type(tValue)) then
			tString = tString .. "S" .. strlen(tValue) .. "+" .. tValue;
		elseif ("number" == type(tValue)) then
			tValue = floor(tValue * 1000) * 0.001;
			tString = tString .. "N" .. strlen(tostring(tValue)) .. "+" .. tValue;
		elseif ("boolean" == type(tValue)) then
			if (tValue) then
				tString = tString .. "1";
			else
				tString = tString .. "0";
			end
		elseif ("table" == type(tValue)) then
			local tNewString = VUHDO_serializeTable(tValue);
			tString = tString .. "T" .. strlen(tNewString) .. "+" .. tNewString;
		end
	end

	return tString;
end



--
local function VUHDO_getValueByLength(aString, aGleichPos)
	local tEndPos = strfind(aString, "+", aGleichPos + 2, true);

	if (tEndPos == nil) then
		return nil;
	end

	local tNumBytes = tonumber(strsub(aString, aGleichPos + 2, tEndPos - 1));
	local tValue = strsub(aString, tEndPos + 1, tEndPos + tNumBytes);
	return tEndPos + tNumBytes + 1, tValue;
end



--
function VUHDO_deserializeTable(aString)
	local tIndex = 1;
	local tKeyType, tValueType;
	local tTable = { };
	local tGleichPos;
	local tKey, tValue;

	while (tIndex <= strlen(aString)) do
		tKeyType = strbyte(aString, tIndex);
		tGleichPos = strfind(aString, "=", tIndex + 1, true);

		if (tGleichPos ~= nil) then
			if (tKeyType == 83) then -- S
				tKey = strsub(aString, tIndex + 1, tGleichPos - 1);
				tKey = VUHDO_ABBREV_TO_KEY[tKey] or tKey;
			elseif (tKeyType == 78) then -- N
				tKey = tonumber(strsub(aString, tIndex + 1, tGleichPos - 1));
			else
				tKey = nil;
			end

			tValueType = strbyte(aString, tGleichPos + 1);
			if (83 == tValueType) then -- S
				tIndex, tValue = VUHDO_getValueByLength(aString, tGleichPos);
			elseif (78 == tValueType) then -- N
				tIndex, tValue = VUHDO_getValueByLength(aString, tGleichPos);
				tValue = tonumber(tValue);
			elseif (48 == tValueType) then -- 0
				tValue = false;
				tIndex = tGleichPos + 2;
			elseif (49 == tValueType) then -- 1
				tValue = true;
				tIndex = tGleichPos + 2;
			elseif (84 == tValueType) then -- T
				tIndex, tValue = VUHDO_getValueByLength(aString, tGleichPos);
				tValue = VUHDO_deserializeTable(tValue);
			else
				return tTable;
			end
		else
			return tTable;
		end

		if (tKey ~= nil and tValue ~= nil) then
			tTable[tKey] = tValue;
		end
	end

	return tTable;
end
